#!/usr/bin/perl -w
# $Id: PFIF.pm,v 1.17 2005/09/22 23:46:14 ianb Exp $
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#


package XML::PFIF;
use strict;
use POSIX;

use vars qw($VERSION @channelfields);
$VERSION = '1.0'; # Version of module
@channelfields=qw(title description lastBuildDate link);

=head1 NAME

XML::PFIF - create PFIF (People Finder Interchange Format) XML and RSS files

=head1 DESCRIPTION

XML::PFIF is a Perl module for reading and writing PFIF (People Finder
Interchange Format) XML and RSS files, for the exchange of data about
missing persons.

XML::PFIF currently implements version 1.1 of the PFIF specification.

There are two classes implementing types of records. Person records
(L<XML::PFIF::Person|XML::PFIF::Person>) are for static
information. Note records (L<XML::PFIF::Note|XML::PFIF::Note>) are for
changing information. Each note record belongs to a particular person,
and a person record may have any number of associated note records.

The XML::PFIF class is a parent class to both these classes,
and provides various helper functions used by both classes.    
It is unlikely to be much use by itself.

The XML::PFIF::Reader class provides an interface (via XML::Twig,
XML::Parser and libexpat) to read PFIF XML and RSS files

See L<XML::PFIF::Person>, L<XML::PFIF::Note> and L<XML::PFIF::Reader>
for details on use of this module, along with the files B<testpfif>,
B<testrss>, B<testrssnote> and B<testreader>, and L<pfifcvt(1)>, a
converter script.

=head1 METHODS

=cut    

#don't really need new, don't document?    

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    # FIXME:  not yet used
    # name of scraper or undef if not automated
    # $self->{automated}=shift; 
    
    return $self;
}


=head3 start_xml()

Returns a string containing the XML header that should start all PFIF
XML and RSS files.

Can be called statically (XML::PFIF::start_xml()) or dynamically
($obj->start_xml()).

=cut

sub start_xml
{
    return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
}

=head3 start_rss()

Returns a string containing the xml snippet to start a PFIF RSS feed.

Can be called statically (XML::PFIF::start_rss()) or dynamically
($obj->start_rss()).

=cut

sub start_rss
{
    my $str=<<'EOT';
<rss version="2.0"
     xmlns:pfif="http://zesty.ca/pfif/1.1"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://zesty.ca/pfif/1.1
                         http://zesty.ca/pfif/1.1/pfif-1.1.xsd">
EOT
    return $str;
}

=head3 end_rss()

Returns a string containing the xml snippet to end a PFIF RSS feed.

Can be called statically (XML::PFIF::end_rss()) or dynamically
($obj->end_rss()).

=cut

sub end_rss
{
    return "</rss>\n";
}


=head3 start_channel()

Returns a string containing the xml snippet to start a PFIF RSS
channel. Take a reference to a hash containing three RSS feed
elements: title, description and link.

Can be called statically (XML::PFIF::start_channel()) or dynamically
($obj->start_channel()).

=cut

sub start_channel
{
    my $hash=shift;
    # In case we are called as a class method
    unless(ref $hash eq "HASH")
    {
	$hash=shift;
    }
    my $fields={};
    my $str= "  <channel>\n";
    my $indent="    ";
    for my $field (@channelfields)
    {
	if(defined($hash) && exists($hash->{$field}))
	{
	    $fields->{$field}=$hash->{$field};
	}
	else
	{
	    $fields->{$field}='';
	}

	$str .=  $indent . tag($field,$fields->{$field});
    }
    return $str;
}


=head3 end_channel()

Returns a string containing the xml snippet to end a channel.

Can be called statically (XML::PFIF::end_channel()) or dynamically
($obj->end_channel()).

=cut

sub end_channel
{
    return "  </channel>\n";
}

=head3 start_pfif()

Returns a string containing the xml snippet to start a PFIF
container. Not needed with RSS, as XML::PFIF::Person::to_rss
fills it in automatically.

Can be called statically (XML::PFIF::start_pfif()) or dynamically
($obj->start_pfif()).

=cut

sub start_pfif
{
    my $str=<<'EOT';
<pfif:pfif xmlns:pfif="http://zesty.ca/pfif/1.1"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
           xsi:schemaLocation="http://zesty.ca/pfif/1.1
                               http://zesty.ca/pfif/1.1/pfif-1.1.xsd">
EOT
    return $str;
}

=head3 end_pfif()

Returns a string containing the xml snippet to end a PFIF
container. Not needed with RSS, as XML::PFIF::Person::to_rss fills it
in automatically..

Can be called statically (XML::PFIF::end_pfif()) or dynamically
($obj->end_pfif()).

=cut

sub end_pfif
{
    return "</pfif:pfif>\n";
}

=head3 tag($tag, $value)

Returns a string containing $tag printed as xml, either with $value
filled in or, if empty, as a self-closing <tag/>.

Can be called statically (XML::PFIF::tag()) or dynamically
($obj->tag()).

=cut

sub tag
{
    if(ref($_[0])) { shift; }
    my($tag,$val)=@_;
    my $str='';
    if($val ne '')
    {
	$str="<$tag>$val</$tag>\n";
    }
    else
    {
	$str="<$tag/>\n";
    }
    return $str;
}


=head3 date_to_rfc822($date)

Converts $date which should be in the format "yyyy-mm-ddThh:mm:ssZ"
to a rfc822-compliant date for use in the rss <channel> pubDate attribute.

Can be called statically (XML::PFIF::date_to_rfc822()) or dynamically
($obj->date_to_rfc822()).

=cut

sub date_to_rfc822
{
    my $indate=shift;
    if(ref($indate)) { $indate=shift; }
    if($indate=~/^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	my($year,$month,$day,$hour,$min,$sec)=($1,$2,$3,$4,$5,$6);
	return POSIX::strftime("%a, %d %b %Y %H:%M:%S GMT",
			       $sec, $min, $hour, $day, ($month-1), ($year-1900));
    }
    return '';
}

=head3 now()

Returns the current date and time in "yyyy-mm-ddThh:mm:ssZ"
format.    

=cut

sub now
{
    return POSIX::strftime("%Y-%m-%dT%H:%M:%SZ",gmtime(time()));
}


=head3 date_to_human($date)

Converts $date which should be in the format "yyyy-mm-ddThh:mm:ssZ"
to a human-readable format.

Can be called statically (XML::PFIF::date_to_human()) or dynamically
($obj->date_to_human()).

=cut

sub date_to_human
{
    my $indate=shift;
    if(ref($indate)) { $indate=shift; }
    if($indate=~/^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	my($year,$month,$day,$hour,$min,$sec)=($1,$2,$3,$4,$5,$6);
	return POSIX::strftime("%H:%M:%S GMT on %Y-%m-%d",
			       $sec, $min, $hour, $day, ($month-1), ($year-1900));
    }
    return '';
}


=head3 validate_date($date)

Returns 1 if the date is in a valid "yyyy-mm-ddThh:mm:ssZ"
format, else warn()s of the error and returns 0;

Can be called statically (XML::PFIF::validate_date()) or dynamically
($obj->validate_date()).

=cut

sub validate_date
{
    my $date=shift;
    if(ref($date)) { $date=shift; }
    if($date=~/^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	my($years,$months,$days,$hours,$mins,$secs)=($1,$2,$3,$4,$5,$6);
	if(($months < 1) || ($months > 12))
	{
	    warn("validate_date: bad month: $months\n");
	    return 0;
	}
	if(($days < 1) || ($days > 31))
	{
	    warn("validate_date: bad day: $days\n");
	    return 0;
	}
	if(($hours < 0) || ($hours > 24))
	{
	    warn("validate_date: bad hour: $hours\n");
	    return 0;
	}
	if(($days < 1) || ($days > 31))
	{
	    warn("validate_date: bad day: $days\n");
	    return 0;
	}
	if(($mins < 0) || ($mins > 60))
	{
	    warn("validate_date: bad minute: $mins\n");
	    return 0;
	}
	if(($secs < 0) || ($secs > 60))
	{
	    warn("validate_date: bad second: $secs\n");
	    return 0;
	}
    }
    else
    {
	warn("validate_date: bad date format: expecting yyyy-mm-ddThh:mm:ssZ\n");
	return 0;
    }
	     
    return 1;
}

=head3 fixup_string($string)

Converts $string to the canonical format for searchable fields.
Accents are converted to their non-accent equivalents, and letters are
upcased.

Can be called statically (XML::PFIF::fixup_string()) or dynamically
($obj->fixup_string()).

=cut

sub fixup_string
{
    my $str=shift;
    if(ref($str)) { $str=shift; }

    my $achars="";
    my $aechars="";
    my $bchars="";
    my $cchars="";
    my $echars="";
    my $ichars="";
    my $nchars="";
    my $ochars="";
    my $uchars="";
    my $ychars="";

    $str=~s/[$achars]/A/g;
    $str=~s/[$aechars]/AE/g;
    $str=~s/[$bchars]/B/g;
    $str=~s/[$cchars]/C/g;
    $str=~s/[$echars]/E/g;
    $str=~s/[$ichars]/I/g;
    $str=~s/[$nchars]/N/g;
    $str=~s/[$ochars]/O/g;
    $str=~s/[$uchars]/U/g;
    $str=~s/[$ychars]/Y/g;

    $str=~tr/[a-z]/[A-Z]/;

    # fix escaped &<> Is this sufficient or will other things crop up?
    $str=~s/&AMP;/&amp;/g;
    $str=~s/&LT;/&lt;/g;
    $str=~s/&GT;/&gt;/g;

    return $str;

}

=head3 is_complete()

Returns 1 if record has all mandatory fields filled in.

Can NOT be called statically, must be called via an object
($obj->is_complete()).

=cut

sub is_complete
{
    my ($self)=@_;
    for my $field (@{$self->{mandatory_fields}})
    {
	if($self->{$field} eq '')
	{
	    return 0;
	}
    }
    return 1;
}

=head3 escape(str)

Escapes a string for storing as xml.

Converts '&', '<' and '>' to &amp;, &lt; and &gt;
respectively.

Can be called statically (XML::PFIF::escape()) or dynamically
($obj->escape()).

=cut

sub escape
{
    my $str=shift;
    if(ref $str) { $str=shift; }
    $str=~s/\&/&amp;/g;
    $str=~s/\</&lt;/g;
    $str=~s/\>/&gt;/g;
    return $str;
}

=head3 unescape(str)

Unescapes a string retrieved from an XML document.
Not needed for XML::PFIF::Reader, as the underlying    
XML::Twig module handles this automatically, but included
for completeness.    

Converts &amp;, &lt; and &gt; to '&', '<' and '>' respectively.

Can be called statically (XML::PFIF::unescape()) or dynamically
($obj->unescape()).

=cut

sub unescape
{
    my $str=shift;
    if(ref $str) { $str=shift; }
    $str=~s/\&amp\;/&/ig;
    $str=~s/\&lt\;/</ig;
    $str=~s/\&gt\;/>/ig;
    return $str;
}

=head3 valid_state(str)

Validates a two-letter state code. Returns 1 if is an existing state,
0 otherwise.

Can be called statically (XML::PFIF::valid_state()) or dynamically
($obj->valid_state()).

=cut

sub valid_state
{
    my $str=shift;
    if(ref $str) { $str=shift; }

    my @state_codes=("AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE",
		     "FL", "GA", "HI", "ID", "IL", "IN", "IA", "KS",
		     "KY", "LA", "ME", "MD", "MA", "MI", "MN", "MS",
		     "MO", "MT", "NE", "NV", "NH", "NJ", "NM", "NY",
		     "NC", "ND", "OH", "OK", "OR", "PA", "RI", "SC",
		     "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV",
		     "WI", "WY", "AS", "DC", "FM", "GU", "MH", "MP",
		     "PW", "PR", "VI", "AE", "AA", "AE", "AE", "AE",
		     "AP", "AB", "BC", "MB", "NB", "NL", "NS", "NT",
		     "NU", "ON", "PE", "QC", "SK", "YT");
    $str=~tr/[a-z]/[A-Z]/;
    if(grep($str eq $_, @state_codes))
    {
	return 1;
    }
    return 0;
}

1;

__END__


=head1 BUGS

Please report any found to ianb@nessie.mcc.ac.uk	

=head1 EXAMPLES

See B<testpfif>, B<testrss>, B<testrssnote> and B<testreader> with the
XML::PFIF distribution, which (mostly) reproduce the example XML and
RSS files at L<http://zesty.ca/pfif/>.

=head1 SEE ALSO    

L<XML::PFIF::Person>, L<XML::PFIF::Note>, L<XML::PFIF::Reader>, L<pfifcvt(1)>

PFIF Specification: L<http://zesty.ca/pfif/>

Katrina PeopleFinder Project wiki Tech page: L<http://katrina.internet2.edu/wiki/>

=head1 AUTHOR

Ian Beckwith <ianb@nessie.mcc.ac.uk>

=head1 AVAILABILITY

The latest version can be found at:

B<http://nessie.mcc.ac.uk/~ianb/projects/pfif/>

=head1 COPYRIGHT

Copyright 2005 Ian Beckwith <ianb@nessie.mcc.ac.uk>

This code is in the Public Domain.

If you need a license other than Public Domain,
then consider it under the BSD License:

 Copyright (c) Ian Beckwith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The names of its contributors may not be used to endorse or
    promote products derived from this software without specific prior
    written permission.

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

=cut


