#!/usr/bin/perl -w
# $Id: Note.pm,v 1.16 2005/09/22 23:46:14 ianb Exp $
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#

use XML::PFIF;
package XML::PFIF::Note;
@ISA=qw(XML::PFIF);
use strict;

=head1 NAME

XML::PFIF::Note - handle Note records in PFIF (People Finder Interchange Format)

=head1 SYNOPSIS

 use XML::PFIF;
 use XML::PFIF::Person;
 use XML::PFIF::Note;

 my $note=new XML::PFIF::Note({note_record_id=>$id, entry_date=>$date, ...});
 $note->source_date("2005-09-03T09:21:12Z");

 $note->person($person); # connect it with a person record
 print $note->start_xml();
 print $note->start_rss();
 print $note->start_channel({title=>"SalesForce People Finder RSS Feed for ", $note->person()->first_name() ." ".$note->person->last_name() ,
			 description=> $note->person()->first_name() ." ".$note->person()->last_name() ,
			 lastBuildDate=> $note->now(),
			 link=>"http://www.afallstardjs.com/specificpersonrecord"});

 print $note->to_rss();
 print $note->end_channel();
 print $note->end_rss();

=head1 DESCRIPTION

This modules encapsulates PFIF (People Finder Interchange Format)
note records and provides methods to convert them to XML and RSS.

Each note record belongs to exactly one person record. There may be
any number of note records associated with a particular person record.

=head2 Setting or getting field values    

Fields can be set in a hash passed to new() and/or by calling methods
named for the field name. Where possible, correction and validation of
the input is performed. If the value passed is not valid, the method
returns C<undef>, otherwise it returns the new value.  Calling the
method with no parameters also returns the current value.

For example:

    die("date validation failure") unless($note->source_date($date));
    print "source_date: ",$note->source_date(),"\n";

=head2 Generating XML

To the best of my knowledge, the spec doesn't mention anything about
XML feeds of just notes. If you want to do this, have a look at the
example code in the corresponding section of L<XML::PFIF::Person>.

=head2 Generating RSS    

To generate an RSS feed of an array of Note objects C<@notes>, along
with associated person objects, you would use code like the following:

 use XML::PFIF;
 use XML::PFIF::Note;

 print XML::PFIF::start_xml();
 print XML::PFIF::start_rss();
 print XML::PFIF::start_channel();
 for my $note (@notes)
 {
   print $note->to_rss();
 }
 print XML::PFIF::end_channel();
 print XML::PFIF::end_rss();

The methods C<start_xml>, C<end_xml>, C<start_rss>, C<end_rss>,
C<start_channel> and C<end_channel> are inherited from L<XML::PFIF>.

This assumes that the notes were either created with C<$person-\>add_note()>
or had C<$note-\>person($person)> called to set the associated person object.

=head1 DATA FIELDS

=head2 Information About a Missing Person That Changes Over Time (11 fields)

The found, email_of_found_person, phone_of_found_person and
last_known_location fields store data that changes over time. When
these fields are present in a note record, the record is specifying
new values for these fields, and the source_date field indicates the
date that the new values took effect. So, for example, an application
that wants to display the most recent known location can look for the
note with the latest source_date that has a non-empty
last_known_location field.

=head3 note_record_id [string]:

Unique identifier for this record, which consists of a domain
name followed by a slash and a local identifier. The domain
name identifies this record's home repository, which is the
authority for this record. The format of the local identifier
is up to the home repository. When the note_record_id begins
with a domain other than the application's own domain, it means
this record is a clone of a record from another source.

No validation is performed.

=cut

sub note_record_id
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{note_record_id}=$newval;
    }
    return $self->{note_record_id};
}

=head3 entry_date [string in the form "yyyy-mm-ddThh:mm:ssZ"]:

Date in UTC that this copy of this record was stored. A PFIF
repository must guarantee that this value increases
monotonically as records are added, so that a client can update
a copy of a repository by querying for all records with an
entry_date greater than or equal to the entry_date of the last
received record.

Format of date is validated.

=cut

sub entry_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{entry_date}=$newval;
	}
	else
	{
	    warn("entry_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{entry_date};
}


=head3 author_name [string]:

The full name of the person who entered this note.

No validation is performed.

=cut

sub author_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_name}=$newval;
    }
    return $self->{author_name};
}

=head3 author_email [string]:

The preferred contact e-mail address of the person who entered
this note.

No validation is performed.

=cut

sub author_email
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_email}=$newval;
    }
    return $self->{author_email};
}

=head3 author_phone [string]:

The preferred contact phone number of the person who entered
this note.

No validation is performed.

=cut

sub author_phone
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_phone}=$newval;
    }
    return $self->{author_phone};
}



=head3 source_date [string in the form "yyyy-mm-ddThh:mm:ssZ"]:

The date in UTC that the original copy of this note was created
in its home repository. In most cases, notes should be sorted
by this field for display.

Format of date is validated.

=cut

sub source_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{source_date}=$newval;
	}
	else
	{
	    warn("source_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{source_date};
}


=head3 found [boolean string, "true" or "false"]:

This value is "true" if the missing person has been personally
contacted or seen, or "false" otherwise. The text field of this
note should describe HOW and WHEN the person was contacted or
seen.

Input is checked.

=cut

sub found
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if(($newval=~/true/i) ||
	   ($newval eq "1")   ||
	   ($newval=~/yes/i))
	{
	    $self->{found}="true";
	}
	elsif(($newval=~/false/i) ||
	      ($newval eq "0")    ||
	      ($newval=~/no/i))
	{
	    $self->{found}="false";
	}
	elsif($newval eq '')
	{
	    $self->{found}='';
	}
	else
	{
	    warn("found: bad value: $newval: expected \"true\" or \"false\"\n");
	    return undef;
	}
    }
    return $self->{found};
}


=head3 email_of_found_person [string]:

The preferred contact e-mail address of the FOUND person. This
field is present ONLY if the person has been FOUND. The text
field of this note should describe HOW the person's contact
information was determined.

No validation is performed.

=cut

sub email_of_found_person
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{email_of_found_person}=$newval;
    }
    return $self->{email_of_found_person};
}

=head3 phone_of_found_person [string]:

The preferred contact phone number of the FOUND person. This
field is present ONLY if the person has been FOUND. The text
field of this note should describe HOW the person's contact
information was determined.

No validation is performed.

=cut

sub phone_of_found_person
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{phone_of_found_person}=$newval;
    }
    return $self->{phone_of_found_person};
}

=head3 last_known_location [string]:

A free-form description of the last known location of the
person being sought, including the city, state, and as much
detail as possible. The text field of this note should describe
HOW the person's location was determined.

No validation is performed.

=cut

sub last_known_location
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{last_known_location}=$newval;
    }
    return $self->{last_known_location};
}


=head3 text [large string]:

Free-form text description of the person's current condition,
situation and location details, where they were last seen,
corrections to other information, etc.

No validation is performed.

=cut

sub text
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{text}=$newval;
    }
    return $self->{text};
}

=head1 Methods

=head3 new({ note_record_id=>$id, entry_date=>$date, ...});

Constructor method, returns a reference to an XML::PFIF::Note object.
Can be passed a hash of field names and values to initialise the data fields.
Any fields not set in this hash are initialised to the empty string.

If you want the fields to be validated, include validate=>"1" in the
hash of parameters. If any of the the fields fail validation, undef
will be returned instead of a reference to a Note object.

=cut

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    my $hash=shift;
    my $validate=0;
    
    @{$self->{fields}}=qw(note_record_id entry_date author_name author_email author_phone source_date found email_of_found_person phone_of_found_person last_known_location text);
    @{$self->{mandatory_fields}}=qw(note_record_id author_name source_date text);

    if(defined($hash) && exists($hash->{validate}) && $hash->{validate} eq "1")
    {
	$validate=1;
    }

    for my $field (@{$self->{fields}})
    {
	if(defined($hash) && exists($hash->{$field}))
	{
	    my $ret=$self->$field($hash->{$field}); 
	    if($validate && !defined($ret))
	    {
		$self->$field('');
		return undef;
	    }
	}
	else
	{
	    $self->$field('');
	}
    }

    return $self;
}

=head3 to_xml()

Returns a string containing an XML representation of the Note object.

=cut

sub to_xml
{
    my $self=shift;
    my $str="    <pfif:note>\n";
    my $indent="      ";
    for my $field (@{$self->{fields}})
    {
	next if($self->{$field} eq '' && $self->found() eq "false" &&
		(($field eq "email_of_found_person") ||
		($field eq "phone_of_found_person")));
	if($self->{$field} eq '')
	{
	    $str .= "$indent<pfif:$field/>\n";
	}
	else
	{
	    $str .= "$indent<pfif:$field>$self->{$field}</pfif:$field>\n";
	}
    }
    $str .= "    </pfif:note>\n";
    # Fix indentation of closing tags of fields ending in newline
    # (usually pfif:text or pfif:other I imagine)
    $str=~s/^(<\/pfif:)/      $1/mg;
    return $str;
}

=head3 to_rss()

Returns a string containing a representation of the Note object as an
RSS <item>. This includes a minimal version of the associated Person object,
only including mandatory fields.

=cut

sub to_rss
{
    my($self)=@_;
    my $person=$self->person();
    my $str="    <item>\n";
    my $indent=" " x 4;
    
    my $xml=$person->to_xml({minimal=>1, notes=>$self});
    $xml =~ s/^(\s*<\/?pfif)/  $1/gm;
    $str .= $xml;
    $str .= $indent . $self->tag("guid",$self->note_record_id());
    $str .= $indent . $self->tag("author",$self->author_email() ." (". $self->author_name() .")");
    $str .= $indent . $self->tag("pubDate",$self->date_to_rfc822($person->source_date()));

    $str .= $indent . "<description>\n";
    $str .= "<p>Record entered at ". $self->date_to_human($self->source_date()) ."</p>\n";
    $str .= "<p>Note entered by ". $self->author_name ." (" .$self->author_email() .
	") at " . $self->date_to_human($self->source_date()) .":</p>\n";
    $str .= "<pre>\n";
    $str .= $self->text() . "\n";
    if($self->found() eq 'true')
    {
	$str .= "* Person has been found.\n";
    }
    if($self->email_of_found_person() ne '')
    {
	$str .= "* E-mail address of found person: ". $self->email_of_found_person() ."\n";
    }
    if($self->phone_of_found_person() ne '')
    {
	$str .= "* Phone number of found person: ". $self->phone_of_found_person() ."\n";
    }
    if($self->last_known_location() ne '')
    {
	$str .= "* Last known location: " . $self->last_known_location() ."\n";
    }
    $str .= "</pre>\n";
	    
    $str .= $indent . "</description>\n";
    $str .= "    </item>\n";
    return $str;
}

=head3 person

Sets and returns a reference to this Note's associated Person object.

If you didn't create the note with C<$person-\>add_note()>, and wish to
generate a Note feed, you need to use C<$note-\>person($person)> to associate
the note with its associated person.

=cut

sub person
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{person}=$newval;
    }
    return $self->{person};
}

=head3 dispose()

Frees circular references within the Note object, so perl can clean up
after it.

You need this if you are processing lots of records and need to free
up memory, however if you call dispose() on the parent Person object it
takes care of any associated Note objects for you.

=cut

sub dispose
{
    my $self=shift;
    $self->{person}=undef; # Break circular link    
}

# Actually in PFIF.pm, copied here to avoid confusion 

=head3 is_complete()

Returns 1 if Note has all mandatory fields filled in.

=cut
    
1;
		
__END__

=head2 Other Methods

XML::PFIF::Person is a subclass of XML::PFIF, which contains
several methods which are also used by XML::PFIF::Note.    

See L<XML::PFIF> for details of:

C<start_xml, start_channel, end_channel, start_rss, end_rss,
start_pfif, end_pfif, date_to_rfc822, now, validate_date>, etc...


=head1 DIAGNOSTICS

Currently fields setting methods that validate data report problems
with C<warn()>.  There is probably a better way to report the details
of the problems, but you could always do something cunning with
$SIG{__WARN__) to catch then, see I<perlfunc warn>.

=head1 BUGS

Please report any found to ianb@nessie.mcc.ac.uk	

=head1 EXAMPLES

See B<testpfif>, B<testrss>, B<testrssnote> and B<testreader> with the
XML::PFIF distribution, which (mostly) reproduce the example XML and
RSS files at L<http://zesty.ca/pfif/>.

=head1 SEE ALSO    

L<XML::PFIF>, L<XML::PFIF::Person>, L<XML::PFIF::Reader>, L<pfifcvt(1)>

PFIF Specification: L<http://zesty.ca/pfif/>

Katrina PeopleFinder Project wiki Tech page: L<http://katrina.internet2.edu/wiki/>

=head1 AUTHOR

Ian Beckwith <ianb@nessie.mcc.ac.uk>

=head1 AVAILABILITY

The latest version can be found at:

B<http://nessie.mcc.ac.uk/~ianb/projects/pfif/>

=head1 COPYRIGHT

Copyright 2005 Ian Beckwith <ianb@nessie.mcc.ac.uk>

This code is in the Public Domain.

If you need a license other than Public Domain,
then consider it under the BSD License:

 Copyright (c) Ian Beckwith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The names of its contributors may not be used to endorse or
    promote products derived from this software without specific prior
    written permission.

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

=cut


