#!/usr/bin/perl -w
# $Id: Person.pm,v 1.21 2005/09/22 23:46:14 ianb Exp $
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#

use XML::PFIF;
use XML::PFIF::Note;

package XML::PFIF::Person;
@ISA=qw(XML::PFIF);

use strict;

=head1 NAME

XML::PFIF::Person - handle Person records in PFIF (People Finder Interchange Format)

=head1 SYNOPSIS

 use XML::PFIF;
 use XML::PFIF::Person;
 use XML::PFIF::Note;

 my $person=new XML::PFIF::Person({person_record_id=>$id, entry_date=>$date, ...});
 $person->source_date("2005-09-03T09:21:12Z");
 $person->add_note({note_record_id=>$nid, entry_date=>$ndate, ...}); 

 print $person->start_xml();
 print $person->start_pfif();
 print $person->to_xml();
 print $person->end_pfif();

 # or
 print $person->start_xml();
 print $person->start_rss();
 print $person->start_channel({title=>"SalesForce People Finder RSS Feed",
			 description=>"",
			 lastBuildDate=> $p->now(),
			 link=>"http://www.afallstardjs.com/salesforce-feed.php"});

 print $person->to_rss();
 print $person->end_channel();
 print $person->end_rss();

=head1 DESCRIPTION

This modules encapsulates PFIF (People Finder Interchange Format)
person records and provides methods to convert them to XML and RSS.

A person record contains 17 fields. There may be multiple person
records for the same person. In fact, any given application that
imports data from multiple sources is likely to have multiple person
records for the same person. It is up to the application to associate
such records (see the Database Schema section). It is recommended that
applications keep copies of all the records, and separately keep track
of which records correspond to the same person.

=head2 Setting or getting field values    

Fields can be set in a hash passed to new() and/or by calling methods
named for the field name. Where possible, correction and validation of
the input is performed. If the value passed is not valid, the method
returns C<undef>, otherwise it returns the new value.  Calling the
method with no parameters also returns the current value.

For example:

    die("home state validation failure") unless($person->home_state($newstate));
    print "home state: ",$person->home_state(),"\n";

=head2 Generating XML

To generate an XML PFIF feed of an array of Person objects C<@people>,
you would use code like the following:

 my $p=$people[0];
 print $p->start_xml();
 print $p->start_pfif();
 for my $person (@people)
 {
   print $person->to_xml();
 }
 print $p->end_pfif;

The methods C<start_xml>, C<end_xml>, C<start_pfif> and C<end_pfif> are
inherited from L<XML::PFIF>.

=head2 Generating RSS Person Feeds

To generate an RSS Person feed of an array of Person objects
C<@people>, you would use code like the following:

 my $p=$people[0];
 print $p->start_xml();
 print $p->start_rss();
 print $p->start_channel();
 for my $person (@people)
 {
   print $person->to_rss();
 }
 print $p->end_channel();
 print $p->end_rss();

The methods C<start_xml>, C<end_xml>, C<start_rss>, C<end_rss>,
C<start_channel> and C<end_channel> are inherited from L<XML::PFIF>.

=head1 DATA FIELDS

=head2 Static Tracking Information About the Record Itself (8 fields)

=head3 person_record_id [string]:

Unique identifier for this record, which consists of a domain
name followed by a slash and a local identifier. The domain
name identifies this record's home repository, which is the
authority for this record. The format of the local identifier
is up to the home repository. When the person_record_id begins
with a domain other than the application's own domain, it means
this record is a clone of a record from another source.

Currently no validation performed on this method.				     

=cut

sub person_record_id
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{person_record_id}=$newval;
    }
    return $self->{person_record_id};
}

=head3 entry_date [string in the form "yyyy-mm-ddThh:mm:ssZ"]:

Date in UTC that this copy of this record was stored. A PFIF
repository must guarantee that this value increases
monotonically as records are added, so that a client can update
a copy of a repository by querying for all records with an
entry_date greater than or equal to the entry_date of the last
received record.

Validates that data is in the correct format and returns C<undef>
if not.

=cut

sub entry_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{entry_date}=$newval;
	}
	else
	{
	    warn("entry_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{entry_date};
}

=head3 author_name [string]:

The full name of the person who entered this record.

No validation is performed

=cut

sub author_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_name}=$newval;
    }
    return $self->{author_name};
}

=head3 author_email [string]:

The preferred contact e-mail address of the person who entered
this record.

No validation is performed

=cut    

sub author_email
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_email}=$newval;
    }
    return $self->{author_email};
}

=head3 author_phone [string]:

The preferred contact phone number of the person who entered
this record.

No validation is performed (yet).

=cut

sub author_phone
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{author_phone}=$newval;
    }
    return $self->{author_phone};
}

=head3 source_name [string]:

The name of the home repository of this record.

No validation is performed.

=cut

sub source_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{source_name}=$newval;
    }
    return $self->{source_name};
}

=head3 source_date [string in the form "yyyy-mm-ddThh:mm:ssZ"]:

The date in UTC that the original copy of this record was
created in its home repository.

Validates that date is in correct format.

=cut

sub source_date
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '' || $self->validate_date($newval))
	{
	    $self->{source_date}=$newval;
	}
	else
	{
	    warn("source_date: bad date format (must be 'yyyy-mm-ddThh:mm:ssZ'): $newval\n");
	    return undef;
	}
    }
    return $self->{source_date};
}

=head3 source_url [string]:

The URL to this record in its home repository (as specific as
possible, down to the URL of the individual record).

No validation is done (yet).

=cut

sub source_url
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{source_url}=$newval;
    }
    return $self->{source_url};
}

=head2    Static Identifying Information About a Missing Person (9 fields)

These fields are specifically for identifying the person and should be
for data that never changes. These are the fields to search on.
Insisting on all capitals and no accents is ugly, but it makes
searches more likely to converge on the correct record. The other
field is a very crude way to import foreign data, but the formatting
guidelines should make it possible to extract the data again if there
is a desperate need to do so. For other, free-form text was chosen
instead of XML to make it easy for an application to display other
directly in the UI.

=head3 first_name [string, all capitals, no accents]:

First name of the person sought or found, optionally followed
by a space and any middle names or middle initials.

Uppercased, accents converted to their non-accented equivalents.

=cut    

sub first_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{first_name}=$self->fixup_string($newval);
    }
    return $self->{first_name};
}

=head3 last_name [string, all capitals, no accents]:

Last name of the person sought or found.

Uppercased, accents converted to their non-accented equivalents.

=cut    

sub last_name
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{last_name}=$self->fixup_string($newval);
    }
    return $self->{last_name};
}

=head3 home_city [string, city name, all capitals, no accents]:

Home city of the person sought or found.

Uppercased, accents converted to their non-accented equivalents.

=cut

sub home_city
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{home_city}=$self->fixup_string($newval);
    }
    return $self->{home_city};
}

=head3 home_state [string, two-letter postal abbreviation]:

Home state of the person sought or found.

Uppercased, checked it is a 2-letter code.
Not currently checked against a valid state list.

=cut

sub home_state
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval ne '')
	{
	    $newval=~tr/[a-z]/[A-Z]/;
	    unless($self->valid_state($newval))
	    {
		warn("home_state: $newval: not a valid 2-letter state code\n");
		return undef;
	    }
	}
	$self->{home_state}=$newval;
    }
    return $self->{home_state};
}

=head3 home_neighborhood [string, all capitals, no accents]:

Name of the home neighborhood of the person sought or found.

Uppercased, accents converted to their non-accented equivalents.

=cut

sub home_neighborhood
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{home_neighborhood}=$self->fixup_string($newval);
    }
    return $self->{home_neighborhood};
}

=head3 home_street [string, all capitals, no accents]:

Street name (no number) of the home address of the person
sought or found.

Uppercased, accents converted to their non-accented equivalents,
numbers removed.

=cut

sub home_street
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{home_street}=$self->fixup_string($newval);
	# remove numbers also
	$self->{home_street}=~s/^\s*\d+\s+//;
    }
    return $self->{home_street};
}

=head3 home_zip [integer]:

Zip code of the home address of the person sought or found.

Checks if it is just digits.

=cut

sub home_zip
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	if($newval eq '')
	{
	    $self->{home_zip}='';
	}
	elsif($newval =~ /^\s*(\d+)\s*$/)
	{
	    $self->{home_zip}=$1;
	}
	else
	{
	    warn("home_zip: $newval: not a valid zip code\n");
	    return undef;
	}
    }
    return $self->{home_zip};
}

=head3 photo_url [string]:

URL to an image of an identifying photograph of the person
sought or found.

No validation performed (yet)    

=cut

sub photo_url
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{photo_url}=$newval;
    }
    return $self->{photo_url};
}

=head3 other [large string]:

Free-form text containing any other static data fields brought
in from other sources. (Non-static data imported from other
sources should go into a note record.) Short fields should be
on a single line with the field name, a colon, and the field
value. Long fields can be given as a line with the field name
and a colon, then text indented on the following lines. When a
record is converted from some other form to PFIF by a machine
process, the field "automated-pfif-author" should be present
and should name the program that produced the PFIF. The
"automated-pfif-author" field is not added when records are
exported from a PFIF repository. A description of the person in
free-form text can also go here, with the field name
"description". For example, a program that scrapes a record
from a non-PFIF format that includes a free-form text field
might produce an other field like this:

 description:
     Dark hair, in her late thirties.
     Also goes by the names "Kate" or "Katie".
 automated-pfif-author: ScrapeMatic 0.5

Field names for data fields imported from other applications
should begin with the domain name and a slash. For example, if
a birthdate is imported from an ICRC record, it might look like
this:

 icrc.org/birthdate: 1976-02-26

Two methods are provided here, B<other> and B<add_other>

C<other()> and C<add_other()>. C<other()> is a standard set/get method
like the other fields. C<add_other()> takes a key and value and appends
them to the existing 'other' data.

No validation is performed.

=cut

sub other
{
    my ($self,$newval)=(@_);
    if(defined($newval))
    {
	$self->{other}=$newval;
    }
    return $self->{other};
}

sub add_other
{
    my ($self,$newkey,$newval)=(@_);
    if(defined($newkey) && defined($newval))
    {
	$self->{other} .= "$newkey:";
	if( (length($newkey)+length($newval)) > 80 )
	{
	    $self->{other} .= "\n";
	}
	else
	{
	    $self->{other} .= " ";
	}

	$self->{other} .= $newval;
    }
    return $self->{other};
}

=head1 Methods

=head3 new({ person_record_id=>$id, entry_date=>$date, ...});

Constructor method, returns a reference to an XML::PFIF::Person object.
Can be passed a hash of field names and values to initialise the data fields.
Any fields not set in this hash are initialised to the empty string.

If you want the fields to be validated, include validate=>"1" in the
hash of parameters. If any of the the fields fail validation, undef
will be returned instead of a reference to a Person object.

=cut

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    my $hash=shift;
    my $validate=0;
    $self->{notes}=();
    @{$self->{fields}}=qw(person_record_id entry_date author_name author_email author_phone source_name source_date source_url first_name last_name home_city home_state home_neighborhood home_street home_zip photo_url other);
    @{$self->{mandatory_fields}}=qw(person_record_id first_name last_name);
    
    if(defined($hash) && exists($hash->{validate}) && $hash->{validate} eq "1")
    {
	$validate=1;
    }

    for my $field (@{$self->{fields}})
    {
	if(defined($hash) && exists($hash->{$field}))
	{
	    my $ret=$self->$field($hash->{$field});
	    if($validate && !defined($ret))
	    {
		$self->$field('');
		return undef;
	    }
	}
	else
	{
	    $self->$field('');
	}
    }
    return $self;
}

=head3 to_xml()

Returns a string containing an XML representation of the Person object
(and any associated Note objects). Pass a hashref containing
minimal=>1 to only include mandatory fields.

=cut

sub to_xml
{
    my $self=shift;
    my $hash=shift;
    my @notes=$self->notes();
    my @fields=@{$self->{fields}};
    if(defined($hash) && exists($hash->{minimal}) && $hash->{minimal} eq "1")
    {
	@fields=@{$self->{mandatory_fields}};
    }
    if(defined($hash) && exists($hash->{notes}))
    {
	@notes=$hash->{notes};
    }
    my $str="  <pfif:person>\n";
    for my $field (@fields)
    {
	if($self->{$field} eq '')
	{
	    $str .= "    <pfif:$field/>\n";
	}
	else
	{
	    $str .= "    <pfif:$field>$self->{$field}</pfif:$field>\n";
	}
    }
    # Fix indentation of closing tags of fields ending in newline
    # (usually pfif:text or pfif:other I imagine)
    $str=~s/^(<\/pfif:)/    $1/mg;

    for my $note (@notes)
    {
	$str .= $note->to_xml();
    }

    $str .= "  </pfif:person>\n";

    return $str;
}

=head3 to_rss()

Returns a string containing a representation of the Person object
(and any associated Note objects) as an RSS <item>. Takes care
of opening and closing pfif:pfif tags for you.

=cut    

sub to_rss
{
    my($self)=@_;
    my $str="    <item>\n      <pfif:pfif>\n";
    my $indent=" " x 6 ;
    my $xml=$self->to_xml();
    $xml =~ s/^(\s*<\/?pfif)/$indent$1/gm;
    $str .= $xml;
    $str .="      </pfif:pfif>\n";
    $str .= $indent . $self->tag("guid",$self->person_record_id());
    $str .= $indent . $self->tag("title",$self->first_name() ." ". $self->last_name());
    $str .= $indent . $self->tag("author",$self->author_email() ." (". $self->author_name() .")");
    $str .= $indent . $self->tag("pubDate",$self->date_to_rfc822($self->source_date()));

    $str .= $indent . "<description>\n";
    $str .= "<p>Person sought:</p>\n";
    if($self->photo_url() ne '')
    {
	$str .="<p><img src=\"". $self->photo_url() . "\" /></p>\n";
    }
    $str .= "<pre>\n";
    $str .= $self->first_name() . " " . $self->last_name() . "\n";
    $str .= $self->home_street() ."\n";
    $str .= $self->home_neighborhood() . "\n";
    $str .= $self->home_city() . ", " . $self->home_state ." ".
	    $self->home_zip ."\n\n";
    $str .= $self->other();
    $str .= "</pre>\n";
    $str .= "<p>Record entered by:</p>\n";
    $str .= "<pre>\n";
    $str .= $self->author_name() ."\n";
    $str .= $self->author_email() ."\n";
    $str .= $self->author_phone() ."\n";
    $str .= "</pre>\n";
    $str .= "<p>Record entered at ". $self->date_to_human($self->source_date()) . ".</p>\n";


    for my $note ($self->notes())
    {
	$str .= "<p>Note entered by ". $note->author_name ." (" .$note->author_email() .
	    ") at " . $self->date_to_human($note->source_date()) .":</p>\n";
	$str .= "<pre>\n";
	$str .= $note->text() . "\n";
	if($note->found() eq 'true')
	{
	    $str .= "* Person has been found.\n";
	}
	if($note->email_of_found_person() ne '')
	{
	    $str .= "* E-mail address of found person: ". $note->email_of_found_person() ."\n";
	}
	if($note->phone_of_found_person() ne '')
	{
	    $str .= "* Phone number of found person: ". $note->phone_of_found_person() ."\n";
	}
	if($note->last_known_location() ne '')
	{
	    $str .= "* Last known location: " . $note->last_known_location() ."\n";
	}
	$str .= "</pre>\n";
    }

    $str .= $indent . "</description>\n";
    $str .= $indent . $self->tag("source",$self->source_name());
    $str .= $indent . $self->tag("link",$self->source_url());
    $str .= "    </item>\n";
    return $str;
}

=head3 add_note()

Creates and returns a new XML::PFIF::Note object attached to the
current Person.  Passes arguments to the Note object's new(), which
means you can initialise the note's fields by passing a hashref to
C<add_note()>. See L<XML::PFIF::Note>.

=cut

sub add_note
{
    my ($self,@args)=@_;
    my $note=new XML::PFIF::Note(@args);
    unless(defined($note)) { return undef; }
    $note->person($self);
    push(@{$self->{notes}},$note);
    return $note;
}

=head3 notes()

Returns the notes corresponding to this person as a list of
XML::PFIF::Note objects. Returns the notes sorted by source_date,
earliest first.

=cut

sub notes
{
    my $self=shift;
    return (reverse sort sort_notes (@{$self->{notes}}));
}

=head3 dispose()

Frees circular references within the Person object and associated
Notes, so perl can clean up after it.

You need this if you are processing lots of records and need to free
up memory.

=cut

sub dispose
{
    my $self=shift;
    for my $note ($self->notes())
    {
	$note->dispose();
    }
    $self->{notes}=();
}

=head3 is_complete()

Returns 1 if Person and associated Notes have all mandatory fields
filled in, otherwise returns 0.

=cut

sub is_complete
{
    my ($self)=@_;
    # check notes
    for my $note($self->notes())
    {
	unless($note->is_complete())
	{
	    return 0;
	}
    }
    # check person itself
    return $self->SUPER::is_complete();
}

=head3 sort_notes

Sort notes by source_date.

=cut    

sub sort_notes($$)
{
    my($a,$b)=@_;
    my($ayears,$amonths,$adays,$ahours,$amins,$asecs);
    my($byears,$bmonths,$bdays,$bhours,$bmins,$bsecs);

    my $adate=$a->source_date();
    my $bdate=$b->source_date();

    my $fail=0;

    unless(defined($adate)) { $fail=1; }
    unless(defined($bdate)) { $fail=1; }
    
    if($adate =~ /^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	($ayears,$amonths,$adays,$ahours,$amins,$asecs)=($1,$2,$3,$4,$5,$6);
    }
    else { $fail=1; }
    if($bdate =~ /^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	($byears,$bmonths,$bdays,$bhours,$bmins,$bsecs)=($1,$2,$3,$4,$5,$6);
    }
    else { $fail=1; }
    if($fail) { print STDERR "FAIL!\n";return 0; }
    unless(($ayears <=> $byears) == 0) { return ($ayears <=> $byears); }
    unless(($amonths <=> $bmonths) == 0) { return ($amonths <=> $bmonths); }
    unless(($adays <=> $bdays) == 0) { return ($adays <=> $bdays); }
    unless(($ahours <=> $bhours) == 0) { return ($ahours <=> $bhours); }
    unless(($amins <=> $bmins) == 0) { return ($amins <=> $bmins); }
    return ($asecs <=> $bsecs);
}


1;

__END__

=head2 Other Methods

XML::PFIF::Person is a subclass of XML::PFIF, which contains
several methods which are also used by XML::PFIF::Note.    

See L<XML::PFIF> for details of:

C<start_xml, start_channel, end_channel, start_rss, end_rss,
start_pfif, end_pfif, date_to_rfc822, now, validate_date>, etc...


=head1 DIAGNOSTICS

Currently fields setting methods that validate data report problems
with C<warn()>.  There is probably a better way to report the details
of the problems, but you could always do something cunning with
$SIG{__WARN__) to catch then, see I<perlfunc warn>.

=head1 BUGS

Please report any found to ianb@nessie.mcc.ac.uk	

=head1 EXAMPLES

See B<testpfif>, B<testrss>, B<testrssnote> and B<testreader> with the
XML::PFIF distribution, which (mostly) reproduce the example XML and
RSS files at L<http://zesty.ca/pfif/>.

=head1 SEE ALSO    

L<XML::PFIF>, L<XML::PFIF::Note>, L<XML::PFIF::Reader>, L<pfifcvt(1)>

PFIF Specification: L<http://zesty.ca/pfif/>

Katrina PeopleFinder Project wiki Tech page: L<http://katrina.internet2.edu/wiki/>


=head1 AUTHOR

Ian Beckwith <ianb@nessie.mcc.ac.uk>

=head1 AVAILABILITY

The latest version can be found at:

B<http://nessie.mcc.ac.uk/~ianb/projects/pfif/>

=head1 COPYRIGHT

Copyright 2005 Ian Beckwith <ianb@nessie.mcc.ac.uk>

This code is in the Public Domain.

If you need a license other than Public Domain,
then consider it under the BSD License:

 Copyright (c) Ian Beckwith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The names of its contributors may not be used to endorse or
    promote products derived from this software without specific prior
    written permission.

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

=cut
