# $Id: Reader.pm,v 1.5 2005/09/15 20:48:47 ianb Exp $
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#

use XML::PFIF;
use XML::PFIF::Person;
use XML::PFIF::Note;

unless(defined(eval { require XML::Twig; }))
{
    die("XML::Twig not found - Reading XML not supported\nError:\n$@");
}

package XML::PFIF::Reader;
#@ISA=qw(XML::PFIF);
use strict;

=head1 NAME

XML::PFIF::Reader - read streams of PFIF (People Finder Interchange Format) XML and RSS

=head1 SYNOPSIS

 use XML::PFIF;
 use XML::PFIF::Person;
 use XML::PFIF::Reader;

 my $reader=new XML::PFIF::Reader({file=>$file, callback=>\&do_person});

 print XML::PFIF::start_xml();
 print XML::PFIF::start_pfif();

 if(!$reader->parse())
 {
     warn("$me: $file: XML parse error\n");
 }

 print XML::PFIF::end_pfif();


 sub do_person
 {
    my $person=shift;
    print $person->to_xml();
    return 1;
 }

=head1 DESCRIPTION

This modules reads PFIF (People Finder Interchange Format) XML and RSS
files and calls a user supplied callback with each Person record found
(incorporating any attached Note records).

=head1 METHODS

=head3 new({ file=>$file, callback=>\&callback});

Constructor method, returns a reference to an XML::PFIF::Reader object.

Takes a hashref containing two arguments, B<file> and B<callback>.

B<file> is the file to read. This can contain PFIF as straight XML
or embedded in an RSS feed.

B<callback> is a reference to a user-supplied function which is called once
with each Person record read.

For example:

 my $reader=new XML::PFIF::Reader({file=>$file,
				   callback=> sub { my $p=shift;
						    print $p->to_xml(); } });

=cut

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    my $hash=shift;

    unless((defined($hash) && exists($hash->{file})))
    {
	warn("XML::PFIF::Reader::new : no file parameter suppplied\n");
	return undef;
    }
    
    unless((defined($hash) && exists($hash->{callback})))
    {
	warn("XML::PFIF::Reader::new : no callback parameter suppplied\n");
	return undef;
    }
    $self->{file}=$hash->{file};
    $self->{callback}=$hash->{callback};

    $self->{twig}=new XML::Twig(twig_handlers => { "pfif:person" => sub { return (XML::PFIF::Reader::handle_person($self,@_)) } } );

    return $self;
}

=head3 parse()

Parses an XML file and calls the user supplied callback for each
Person record found. The filename and callback are set in new().

=cut

sub parse
{
    my ($self)=@_;
    my $ret=$self->{twig}->parsefile($self->{file},  ErrorContext => 2);
    if(!$ret)
    {
	$self->{twig}->xpcarp("XML Error:");
    }
    $self->{twig}->dispose();
    return $ret;
}

=head2 Internal Methods

=head3 handle_person($twig,$element)

Callback function passed to XML::Twig. Populates an XML::PFIF::Person
object (including any attached Notes) and passes it to the user-supplied
callback function.

=cut    

sub handle_person
{
    my ($self,$twig,$element)=(@_);
    my @personfields=qw(person_record_id entry_date author_name author_email author_phone source_name source_date source_url first_name last_name home_city home_state home_neighborhood home_street home_zip photo_url other);
    my @notefields=qw(note_record_id entry_date author_name author_email author_phone source_date found email_of_found_person phone_of_found_person last_known_location text);

    my @elements=$element->children();
    my %personhash=$self->copyfields(\@personfields,\@elements);
    my $person=new XML::PFIF::Person({%personhash});

    my @notes=grep { $_->name() eq 'pfif:note' } @elements;
    for my $note (@notes)
    {
	my @noteelements=$note->children();
	my %notehash=$self->copyfields(\@notefields,\@noteelements);
	$person->add_note({%notehash});
    }
    $self->{callback}($person);
    $twig->purge();
    return 1;
}

=head3 copyfields(\@fields,\@twigelements)

Extract the named fields from twigelements and return a hash of field
names and values.

=cut

sub copyfields
{
    my ($self,$fields,$elements)=@_;
    my %inhash=();
    my %outhash=();

    foreach my $el (@$elements)
    {
	my $field=$el->name();
	$field=~s/pfif://;
	$inhash{$field}=$el->text();
    }

    foreach my $field (@$fields)
    {
	if(exists($inhash{$field}))
	{
	    $outhash{$field}=$inhash{$field};
	}
    }
    return (%outhash);
}


1;

		
__END__

=head1 BUGS

Please report any bugs found to ianb@nessie.mcc.ac.uk	

=head1 EXAMPLES

See pfifcvt(1) and the examples B<testreader>, B<testpfif> and
B<testrss> with the XML::PFIF distribution, which (mostly) reproduce
the example XML and RSS files at L<http://zesty.ca/pfif/>

=head1 SEE ALSO    

L<XML::PFIF>, L<XML::PFIF::Person>, L<XML::PFIF::Note>,

PFIF Specification: L<http://zesty.ca/pfif/>

Katrina PeopleFinder Project wiki Tech page: L<http://katrina.internet2.edu/wiki/>

=head1 AUTHOR

Ian Beckwith <ianb@nessie.mcc.ac.uk>

=head1 AVAILABILITY

The latest version can be found at:

B<http://nessie.mcc.ac.uk/~ianb/projects/pfif/>

=head1 COPYRIGHT

Copyright 2005 Ian Beckwith <ianb@nessie.mcc.ac.uk>

This code is in the Public Domain.

If you need a license other than Public Domain,
then consider it under the BSD License:

 Copyright (c) Ian Beckwith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The names of its contributors may not be used to endorse or
    promote products derived from this software without specific prior
    written permission.

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

=cut


